# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.http import Http404
from django.utils.translation import pgettext_lazy
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import messages
from horizon import tables

from openstack_dashboard import policy
from tacker_horizon.openstack_dashboard import api
from tackerclient.common.exceptions import NotFound


class VNFManagerItem(object):
    def __init__(self, name, description, vnfs, vim, status,
                 stack_status, stack_id, error_reason):
        self.name = name
        self.description = description
        self.vnfs = vnfs
        self.vim = vim
        self.status = status
        self.stack_status = stack_status
        self.id = stack_id
        self.error_reason = error_reason


class VNFManagerItemList(object):
    VNFLIST_P = []

    @classmethod
    def get_obj_given_stack_id(cls, vnf_id):
        for obj in cls.VNFLIST_P:
            if obj.id == vnf_id:
                return obj

    @classmethod
    def add_item(cls, item):
        cls.VNFLIST_P.append(item)

    @classmethod
    def clear_list(cls):
        cls.VNFLIST_P = []


class MyFilterAction(tables.FilterAction):
    name = "myfilter"


class StacksUpdateRow(tables.Row):
    ajax = True

    def can_be_selected(self, datum):
        return datum.status != 'DELETE_COMPLETE'

    def get_data(self, request, stack_id):
        try:
            stack = api.heat.stack_get(request, stack_id)
            if stack.stack_status == 'DELETE_COMPLETE':
                # returning 404 to the ajax call removes the
                # row from the table on the ui
                raise Http404
            item = VNFManagerItemList.get_obj_given_stack_id(stack_id)
            item.status = stack.status
            item.stack_status = stack.stack_status
            return item
        except Http404:
            raise
        except Exception as e:
            messages.error(request, e)
            raise


class VNFUpdateRow(tables.Row):
    ajax = True

    def can_be_selected(self, datum):
        return datum.status != 'DELETE_COMPLETE'

    def get_data(self, request, vnf_id):
        try:
            # stack = api.heat.stack_get(request, stack_id)
            # if stack.stack_status == 'DELETE_COMPLETE':
            # returning 404 to the ajax call removes the
            # row from the table on the ui
            #    raise Http404
            item = VNFManagerItemList.get_obj_given_stack_id(vnf_id)
            vnf_instance = api.tacker.get_vnf(request, vnf_id)

            if not vnf_instance and not item:
                # TODO(NAME) - bail with error
                return None

            if not vnf_instance and item:
                # API failure, just keep the current state
                return item

            vnf = vnf_instance['vnf']
            try:
                vnf_services_str = vnf['attributes']['service_type']
            except KeyError:
                vnf_services_str = ""
            try:
                vnf_desc_str = vnf['description']
            except KeyError:
                vnf_desc_str = ""

            vim = vnf['placement_attr']['vim_name']
            if not item:
                # Add an item entry
                item = VNFManagerItem(vnf['name'], vnf_desc_str,
                                      vnf_services_str, str(vim),
                                      vnf['status'], vnf['status'], vnf['id'],
                                      vnf['error_reason'])
            else:
                item.description = vnf_desc_str
                item.vnfs = vnf_services_str
                item.status = vnf['status']
                item.stack_status = vnf['status']
            return item
        except (Http404, NotFound):
            raise Http404
        except Exception as e:
            messages.error(request, e)
            raise


class DeleteVNF(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            "Terminate VNF",
            "Terminate VNFs",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            "Terminate VNF",
            "Terminate VNFs",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vnf(request, obj_id)


class DeployVNF(tables.LinkAction):
    name = "deployvnf"
    verbose_name = _("Deploy VNF")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:vnfmanager:deployvnf"


class VNFManagerTable(tables.DataTable):
    STATUS_CHOICES = (
        ("ACTIVE", True),
        ("ERROR", False),
    )
    STACK_STATUS_DISPLAY_CHOICES = (
        ("init_in_progress", pgettext_lazy("current status of stack",
                                           "Init In Progress")),
        ("init_complete", pgettext_lazy("current status of stack",
                                        "Init Complete")),
        ("init_failed", pgettext_lazy("current status of stack",
                                      "Init Failed")),
        ("create_in_progress", pgettext_lazy("current status of stack",
                                             "Create In Progress")),
        ("create_complete", pgettext_lazy("current status of stack",
                                          "Create Complete")),
        ("create_failed", pgettext_lazy("current status of stack",
                                        "Create Failed")),
        ("delete_in_progress", pgettext_lazy("current status of stack",
                                             "Delete In Progress")),
        ("delete_complete", pgettext_lazy("current status of stack",
                                          "Delete Complete")),
        ("delete_failed", pgettext_lazy("current status of stack",
                                        "Delete Failed")),
        ("update_in_progress", pgettext_lazy("current status of stack",
                                             "Update In Progress")),
        ("update_complete", pgettext_lazy("current status of stack",
                                          "Update Complete")),
        ("update_failed", pgettext_lazy("current status of stack",
                                        "Update Failed")),
        ("rollback_in_progress", pgettext_lazy("current status of stack",
                                               "Rollback In Progress")),
        ("rollback_complete", pgettext_lazy("current status of stack",
                                            "Rollback Complete")),
        ("rollback_failed", pgettext_lazy("current status of stack",
                                          "Rollback Failed")),
        ("suspend_in_progress", pgettext_lazy("current status of stack",
                                              "Suspend In Progress")),
        ("suspend_complete", pgettext_lazy("current status of stack",
                                           "Suspend Complete")),
        ("suspend_failed", pgettext_lazy("current status of stack",
                                         "Suspend Failed")),
        ("resume_in_progress", pgettext_lazy("current status of stack",
                                             "Resume In Progress")),
        ("resume_complete", pgettext_lazy("current status of stack",
                                          "Resume Complete")),
        ("resume_failed", pgettext_lazy("current status of stack",
                                        "Resume Failed")),
        ("adopt_in_progress", pgettext_lazy("current status of stack",
                                            "Adopt In Progress")),
        ("adopt_complete", pgettext_lazy("current status of stack",
                                         "Adopt Complete")),
        ("adopt_failed", pgettext_lazy("current status of stack",
                                       "Adopt Failed")),
        ("snapshot_in_progress", pgettext_lazy("current status of stack",
                                               "Snapshot In Progress")),
        ("snapshot_complete", pgettext_lazy("current status of stack",
                                            "Snapshot Complete")),
        ("snapshot_failed", pgettext_lazy("current status of stack",
                                          "Snapshot Failed")),
        ("check_in_progress", pgettext_lazy("current status of stack",
                                            "Check In Progress")),
        ("check_complete", pgettext_lazy("current status of stack",
                                         "Check Complete")),
        ("check_failed", pgettext_lazy("current status of stack",
                                       "Check Failed")),
    )
    name = tables.Column("name",
                         link="horizon:nfv:vnfmanager:detail",
                         verbose_name=_("VNF Name"))
    description = tables.Column("description",
                                verbose_name=_("Description"))
    vnfs = tables.Column("vnfs",
                         verbose_name=_("Deployed Services"))
    vim = tables.Column("vim", verbose_name=_("VIM"))
    status = tables.Column("status",
                           hidden=True,
                           status=True,
                           status_choices=STATUS_CHOICES)
    stack_status = tables.Column("stack_status",
                                 verbose_name=_("Status"),
                                 display_choices=STACK_STATUS_DISPLAY_CHOICES)
    error_reason = tables.Column("error_reason",
                                 verbose_name=_("Error Reason"))

    class Meta(object):
        name = "vnfmanager"
        verbose_name = _("VNFManager")
        status_columns = ["status", ]
        row_class = VNFUpdateRow
        table_actions = (DeployVNF, DeleteVNF, MyFilterAction,)



# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from oslo_log import log as logging
from oslo_utils import strutils
from tackerclient.v1_0 import client as tacker_client

from horizon.utils.memoized import memoized  # noqa
from openstack_dashboard.api import base
from openstack_dashboard.contrib.developer.profiler import api as profiler


LOG = logging.getLogger(__name__)

SUPPORTED_VIM_TYPES = (
    ('openstack', 'OpenStack'),
    ('kubernetes', 'Kubernetes')
)

AUTH_METHODS = (
    ('basic', _('Basic')),
    ('bearer_token', _('Bearer Token'))
)

CERT_TRUE_TYPE = 'True'
CERT_FALSE_TYPE = 'False'
CERT_VERIFY_TYPES = (
    (CERT_TRUE_TYPE, _("True")),
    (CERT_FALSE_TYPE, _("False"))
)


@memoized
def tackerclient(request):
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
    c = tacker_client.Client(
        token=request.user.token.id,
        auth_url=base.url_for(request, 'identity'),
        endpoint_url=base.url_for(request, 'nfv-orchestration'),
        insecure=insecure, ca_cert=cacert)
    return c


@profiler.trace
def vnf_list(request, **params):
    LOG.debug("vnf_list(): params=%s", params)
    vnfs = tackerclient(request).list_vnfs(**params).get('vnfs')
    return vnfs


@profiler.trace
def vnfd_list(request, **params):
    LOG.debug("vnfd_list(): params=%s", params)
    vnfds = tackerclient(request).list_vnfds(**params).get('vnfds')
    return vnfds


@profiler.trace
def create_vnfd(request, tosca_body=None, **params):
    LOG.debug("create_vnfd(): params=%s", params)
    vnfd_instance = tackerclient(request).create_vnfd(body=tosca_body)
    return vnfd_instance


@profiler.trace
def create_vnf(request, vnf_arg, **params):
    LOG.debug("create_vnf(): vnf_arg=%s", str(vnf_arg))
    vnf_instance = tackerclient(request).create_vnf(body=vnf_arg)
    return vnf_instance


@profiler.trace
def get_vnfd(request, vnfd_id):
    LOG.debug("vnfd_get(): vnfd_id=%s", str(vnfd_id))
    vnfd = tackerclient(request).show_vnfd(vnfd_id)
    return vnfd


@profiler.trace
def get_vnf(request, vnf_id):
    LOG.debug("vnf_get(): vnf_id=%s", str(vnf_id))
    vnf_instance = tackerclient(request).show_vnf(vnf_id)
    return vnf_instance


@profiler.trace
def delete_vnf(request, vnf_id):
    LOG.debug("delete_vnf():vnf_id=%s", str(vnf_id))
    tackerclient(request).delete_vnf(vnf_id)


@profiler.trace
def delete_vnfd(request, vnfd_id):
    LOG.debug("delete_vnfd():vnfd_id=%s", str(vnfd_id))
    tackerclient(request).delete_vnfd(vnfd_id)


@profiler.trace
def create_vim(request, vim_arg):
    LOG.debug("create_vim(): vim_arg=%s", strutils.mask_password(vim_arg))
    vim_instance = tackerclient(request).create_vim(body=vim_arg)
    return vim_instance


@profiler.trace
def get_vim(request, vim_id):
    LOG.debug("vim_get(): vim_id=%s", str(vim_id))
    vim_instance = tackerclient(request).show_vim(vim_id)
    return vim_instance


@profiler.trace
def delete_vim(request, vim_id):
    LOG.debug("delete_vim():vim_id=%s", str(vim_id))
    tackerclient(request).delete_vim(vim_id)


@profiler.trace
def vim_list(request, **params):
    LOG.debug("vim_list(): params=%s", params)
    vims = tackerclient(request).list_vims(**params).get('vims')
    return vims


@profiler.trace
def events_list(request, resource_id):
    params = {'resource_id': resource_id}
    events = tackerclient(request).list_events(**params).get('events')
    LOG.debug("events_list() params=%s events=%s l=%s", params, events,
              len(events))
    return events


@profiler.trace
def vnffg_list(request, **params):
    LOG.debug("vnffg_list(): params=%s", params)
    vnffgs = tackerclient(request).list_vnffgs(**params).get('vnffgs')
    return vnffgs


@profiler.trace
def vnffgd_list(request, **params):
    LOG.debug("vnffgd_list(): params=%s", params)
    vnffgds = tackerclient(request).list_vnffgds(**params).get('vnffgds')
    return vnffgds


@profiler.trace
def create_vnffgd(request, tosca_body=None, **params):
    LOG.debug("create_vnffgd(): params=%s", params)
    vnffgd_instance = tackerclient(request).create_vnffgd(body=tosca_body)
    return vnffgd_instance


@profiler.trace
def create_vnffg(request, vnffg_arg, **params):
    LOG.debug("create_vnffg(): vnf_arg=%s", str(vnffg_arg))
    vnffg_instance = tackerclient(request).create_vnffg(body=vnffg_arg)
    return vnffg_instance


@profiler.trace
def get_vnffgd(request, vnffgd_id):
    LOG.debug("vnffgd_get(): vnffgd_id=%s", str(vnffgd_id))
    vnffgd = tackerclient(request).show_vnffgd(vnffgd_id)
    return vnffgd


@profiler.trace
def get_vnffg(request, vnffg_id):
    LOG.debug("vnffg_get(): vnffg_id=%s", str(vnffg_id))
    vnffg_instance = tackerclient(request).show_vnffg(vnffg_id)
    return vnffg_instance


@profiler.trace
def delete_vnffg(request, vnffg_id):
    LOG.debug("delete_vnffg():vnffg_id=%s", str(vnffg_id))
    tackerclient(request).delete_vnffg(vnffg_id)


@profiler.trace
def delete_vnffgd(request, vnffgd_id):
    LOG.debug("delete_vnffgd():vnffgd_id=%s", str(vnffgd_id))
    tackerclient(request).delete_vnffgd(vnffgd_id)


@profiler.trace
def create_nsd(request, tosca_body=None, **params):
    LOG.debug("create_nsd(): params=%s", params)
    nsd_instance = tackerclient(request).create_nsd(body=tosca_body)
    return nsd_instance


@profiler.trace
def nsd_list(request, **params):
    LOG.debug("nsd_list(): params=%s", params)
    nsds = tackerclient(request).list_nsds(**params).get('nsds')
    return nsds


@profiler.trace
def get_nsd(request, nsd_id):
    LOG.debug("nsd_get(): nsd_id=%s", str(nsd_id))
    nsd = tackerclient(request).show_nsd(nsd_id)
    return nsd


@profiler.trace
def delete_nsd(request, nsd_id):
    LOG.debug("delete_nsd():nsd_id=%s", str(nsd_id))
    tackerclient(request).delete_nsd(nsd_id)


@profiler.trace
def get_ns(request, ns_id):
    LOG.debug("ns_get(): ns_id=%s", str(ns_id))
    ns_instance = tackerclient(request).show_ns(ns_id)
    return ns_instance


@profiler.trace
def delete_ns(request, ns_id):
    LOG.debug("delete_ns():ns_id=%s", str(ns_id))
    tackerclient(request).delete_ns(ns_id)


@profiler.trace
def ns_list(request, **params):
    LOG.debug("ns_list(): params=%s", params)
    nss = tackerclient(request).list_nss(**params).get('nss')
    return nss


@profiler.trace
def create_ns(request, ns_arg, **params):
    LOG.debug("create_ns(): ns_arg=%s", str(ns_arg))
    ns_instance = tackerclient(request).create_ns(body=ns_arg)
    return ns_instance



# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs

from horizon.utils import memoized

from openstack_dashboard import api

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfcatalog \
    import tabs as nfv_tabs

from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfcatalog \
    import forms as project_forms


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = nfv_tabs.VNFCatalogTabs
    template_name = 'nfv/vnfcatalog/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class OnBoardVNFView(forms.ModalFormView):
    form_class = project_forms.OnBoardVNF
    template_name = 'nfv/vnfcatalog/onboardvnf.html'
    success_url = reverse_lazy("horizon:nfv:vnfcatalog:index")
    modal_id = "onboardvnf_modal"
    modal_header = _("OnBoard VNF")
    submit_label = _("OnBoard VNF")
    submit_url = "horizon:nfv:vnfcatalog:onboardvnf"

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.nova.server_get(self.request,
                                       self.kwargs["instance_id"])
        except Exception:
            exceptions.handle(self.request,
                              _("Unable to retrieve instance."))

    def get_initial(self):
        # return {"instance_id": self.kwargs["instance_id"]}
        return {}

    def get_context_data(self, **kwargs):
        context = super(OnBoardVNFView, self).get_context_data(**kwargs)
        # instance_id = self.kwargs['instance_id']
        # context['instance_id'] = instance_id
        # context['instance'] = self.get_object()
        context['submit_url'] = reverse(self.submit_url)
        return context


class DetailView(tabs.TabView):
    tab_group_class = nfv_tabs.VNFDDetailTabs
    template_name = 'nfv/vnfcatalog/detail.html'
    redirect_url = 'horizon:nfv:vnfcatalog:index'
    page_title = _("VNFD Details: {{ vnfd_id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vnfd = self.get_data()
        context['vnfd'] = vnfd
        context['vnfd_id'] = kwargs['vnfd_id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        vnfd_id = self.kwargs['vnfd_id']

        try:
            template = None
            vnfd = tacker_api.tacker.get_vnfd(self.request, vnfd_id)
            attributes_json = vnfd['vnfd']['attributes']
            template = attributes_json.get('vnfd', None)
            vnfd['template'] = template
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Unable to retrieve details for '
                                'VNFD "%s".') % vnfd_id,
                              redirect=redirect)
            raise exceptions.Http302(redirect)
        return vnfd

    def get_tabs(self, request, *args, **kwargs):
        vnfd = self.get_data()
        return self.tab_group_class(request, vnfd=vnfd, **kwargs)



# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv import utils
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfmanager import tables


class VNFManagerTab(tabs.TableTab):
    name = _("VNFManager Tab")
    slug = "vnfmanager_tab"
    table_classes = (tables.VNFManagerTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_vnfmanager_data(self):
        try:
            tables.VNFManagerItemList.clear_list()
            vnfs = api.tacker.vnf_list(self.request)

            if len(vnfs) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for vnf in vnfs:
                try:
                    vnf_services_str = vnf['attributes']['service_type']
                except KeyError:
                    vnf_services_str = ""
                try:
                    vnf_desc_str = vnf['description']
                except KeyError:
                    vnf_desc_str = ""

                vim = vnf['placement_attr']['vim_name']
                obj = tables.VNFManagerItem(
                    vnf['name'],
                    vnf_desc_str,
                    vnf_services_str,
                    vim,
                    vnf['status'],
                    vnf['status'],
                    vnf['id'],
                    vnf['error_reason'])
                tables.VNFManagerItemList.add_item(obj)
            return tables.VNFManagerItemList.VNFLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get instances')
            exceptions.handle(self.request, error_message)

            return []


class VNFManagerTabs(tabs.TabGroup):
    slug = "vnfmanager_tabs"
    tabs = (VNFManagerTab,)
    sticky = True


class VDUDetailTab(tabs.Tab):
    name = _("VDU Detail")
    slug = "VDU_Details"
    template_name = "nfv/vnfmanager/vdu_details.html"

    def get_context_data(self, request):
        return {'vnf': self.tab_group.kwargs['vnf']}


class VNFEventsTab(tabs.TableTab):
    name = _("Events Tab")
    slug = "events_tab"
    table_classes = (utils.EventsTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_events_data(self):
        try:
            utils.EventItemList.clear_list()
            events = api.tacker.events_list(self.request,
                                            self.tab_group.kwargs['vnf_id'])

            if len(events) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for event in events:
                evt_obj = utils.EventItem(
                    event['id'], event['resource_state'],
                    event['event_type'],
                    event['timestamp'],
                    event['event_details'])
                utils.EventItemList.add_item(evt_obj)
            return utils.EventItemList.EVTLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get events.')
            exceptions.handle(self.request, error_message)
            return []


class VNFDetailsTabs(tabs.TabGroup):
    slug = "VNF_details"
    tabs = (VDUDetailTab, VNFEventsTab)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import messages
from horizon import tables

from openstack_dashboard import policy
from tacker_horizon.openstack_dashboard import api
from tackerclient.common.exceptions import NotFound


class NSManagerItem(object):
    def __init__(self, name, description, vim, status,
                 ns_id, error_reason):
        self.name = name
        self.description = description
        self.vim = vim
        self.status = status
        self.id = ns_id
        self.error_reason = error_reason


class NSManagerItemList(object):
    NSLIST_P = []

    @classmethod
    def get_obj_given_stack_ids(cls, ns_id):
        for obj in cls.NSLIST_P:
            if obj.id == ns_id:
                return obj

    @classmethod
    def add_item(cls, item):
        cls.NSLIST_P.append(item)

    @classmethod
    def clear_list(cls):
        cls.NSLIST_P = []


class MyFilterAction(tables.FilterAction):
    name = "myfilter"


class NSUpdateRow(tables.Row):
    ajax = True

    def can_be_selected(self, datum):
        return datum.status != 'DELETE_COMPLETE'

    def get_data(self, request, ns_id):
        try:
            # stack = api.heat.stack_get(request, stack_id)
            # if stack.stack_status == 'DELETE_COMPLETE':
            # returning 404 to the ajax call removes the
            # row from the table on the ui
            #    raise Http404
            item = NSManagerItemList.get_obj_given_stack_ids(ns_id)
            ns_instance = api.tacker.get_ns(request, ns_id)

            if not ns_instance and not item:
                # TODO(NAME) - bail with error
                return None

            if not ns_instance and item:
                # API failure, just keep the current state
                return item

            ns = ns_instance['ns']
            try:
                ns_desc_str = ns['description']
            except KeyError:
                ns_desc_str = ""

            vim = ns['vim_id']
            if not item:
                # Add an item entry
                item = NSManagerItem(ns['name'], ns_desc_str,
                                     str(vim),
                                     ns['status'], ns['id'],
                                     ns['error_reason'])
            else:
                item.description = ns_desc_str
                item.status = ns['status']
                item.id = ns['id']
            return item
        except (Http404, NotFound):
            raise Http404
        except Exception as e:
            messages.error(request, e)
            raise


class DeleteNS(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            "Terminate NS",
            "Terminate NSs",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            "Terminate NS",
            "Terminate NSs",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_ns(request, obj_id)


class DeployNS(tables.LinkAction):
    name = "deployns"
    verbose_name = _("Deploy NS")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:nsmanager:deployns"


class NSManagerTable(tables.DataTable):
    STATUS_CHOICES = (
        ("ACTIVE", True),
        ("ERROR", False),
    )
    name = tables.Column("name",
                         link="horizon:nfv:nsmanager:detail",
                         verbose_name=_("NS Name"))
    description = tables.Column("description",
                                verbose_name=_("Description"))
    vim = tables.Column("vim", verbose_name=_("VIM"))
    status = tables.Column("status",
                           status=True,
                           status_choices=STATUS_CHOICES)
    error_reason = tables.Column("error_reason",
                                 verbose_name=_("Error Reason"))

    class Meta(object):
        name = "nsmanager"
        verbose_name = _("NSManager")
        status_columns = ["status", ]
        row_class = NSUpdateRow
        table_actions = (DeployNS, DeleteNS, MyFilterAction,)



# Copyright 2015 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv import utils
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfcatalog import tables


class VNFCatalogItem(object):
    def __init__(self, name, description, service_types, vnfd_id):
        self.id = vnfd_id
        self.name = name
        self.description = description
        self.service_types = service_types


class VNFCatalogTab(tabs.TableTab):
    name = _("VNFCatalog Tab")
    slug = "vnfcatalog_tab"
    table_classes = (tables.VNFCatalogTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_vnfcatalog_data(self):
        try:
            catalogs = []
            vnfds = api.tacker.vnfd_list(self.request,
                                         template_source="onboarded")

            if len(vnfds) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for vnfd in vnfds:
                s_types = [s_type for s_type in vnfd['service_types']]
                s_types_string = ""
                if len(s_types) > 0:
                    s_types_string = ', '.join(
                        [str(item) for item in s_types])
                item = VNFCatalogItem(vnfd['name'],
                                      vnfd['description'],
                                      s_types_string, vnfd['id'])
                catalogs.append(item)
            return catalogs
        except Exception:
            self._has_more = False
            error_message = _('Unable to get vnf catalogs')
            exceptions.handle(self.request, error_message)

            return []


class VNFCatalogTabs(tabs.TabGroup):
    slug = "vnfcatalog_tabs"
    tabs = (VNFCatalogTab,)
    sticky = True


class TemplateTab(tabs.Tab):
    name = _("Template")
    slug = "template"
    template_name = ("nfv/vnfcatalog/template.html")

    def get_context_data(self, request):
        return {'vnfd': self.tab_group.kwargs['vnfd']}


class VNFDEventsTab(tabs.TableTab):
    name = _("Events Tab")
    slug = "events_tab"
    table_classes = (utils.EventsTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_events_data(self):
        try:
            utils.EventItemList.clear_list()
            events = api.tacker.events_list(self.request,
                                            self.tab_group.kwargs['vnfd_id'])

            if len(events) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for event in events:
                evt_obj = utils.EventItem(
                    event['id'], event['resource_state'],
                    event['event_type'],
                    event['timestamp'],
                    event['event_details'])
                utils.EventItemList.add_item(evt_obj)
            return utils.EventItemList.EVTLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get events.')
            exceptions.handle(self.request, error_message)
            return []


class VNFDDetailTabs(tabs.TabGroup):
    slug = "VNFD_details"
    tabs = (TemplateTab, VNFDEventsTab)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.conf.urls import url

from tacker_horizon.openstack_dashboard.dashboards.nfv.nscatalog import views

urlpatterns = [
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^onboardns', views.OnBoardNSView.as_view(), name='onboardns'),
    url(r'^(?P<nsd_id>[^/]+)/$', views.DetailView.as_view(), name='detail'),
]



# Copyright 2016 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv import utils  # noqa
from tacker_horizon.openstack_dashboard.dashboards.nfv.vim import tables


class VIMItem(object):
    def __init__(self, name, description, is_default, regions, vim_id,
                 auth_url, user, project, status, vim_type):
        self.id = vim_id
        self.name = name
        self.description = description
        self.is_default = is_default
        self.regions = regions
        self.auth_url = auth_url
        self.user = user
        self.project = project
        self.status = status
        self.type = vim_type


class VIMTab(tabs.TableTab):
    name = _("VIM Tab")
    slug = "vim_tab"
    table_classes = (tables.VIMTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_vim_data(self):
        try:
            instances = []
            vims = api.tacker.vim_list(self.request)

            if len(vims) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for vim in vims:
                auth_cred = vim['auth_cred']
                placement_attr = vim['placement_attr']
                vim_regions = ','.join(placement_attr['regions'])
                user = "<Bearer token>"
                if 'bearer_token' not in auth_cred.keys():
                    user = auth_cred['username'] if auth_cred[
                        'username'] else auth_cred['user_id']
                project_info = vim['vim_project']
                project = project_info['name'] if project_info[
                    'name'] else project_info['id']
                status = vim["status"]
                is_default = vim['is_default']
                vim_type = vim["type"]
                item = VIMItem(name=vim.get('name', ''),
                               description=vim.get('description', ''),
                               is_default=is_default,
                               regions=vim_regions,
                               vim_id=vim.get('id', ''),
                               auth_url=vim.get('auth_url', ''),
                               user=user, project=project, status=status,
                               vim_type=vim_type)
                instances.append(item)
            return instances
        except Exception:
            self._has_more = False
            error_message = _('Unable to fetch vim list')
            exceptions.handle(self.request, error_message)

            return []


class VIMEventsTab(tabs.TableTab):
    name = _("Events Tab")
    slug = "events_tab"
    table_classes = (utils.EventsTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_events_data(self):
        try:
            utils.EventItemList.clear_list()
            events = api.tacker.events_list(self.request,
                                            self.tab_group.kwargs['vim_id'])

            if len(events) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for event in events:
                evt_obj = utils.EventItem(
                    event['id'], event['resource_state'],
                    event['event_type'],
                    event['timestamp'],
                    event['event_details'])
                utils.EventItemList.add_item(evt_obj)
            return utils.EventItemList.EVTLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get events.')
            exceptions.handle(self.request, error_message)
            return []


class VIMTabs(tabs.TabGroup):
    slug = "vim_tabs"
    tabs = (VIMTab,)
    sticky = True


class VIMDetailsTabs(tabs.TabGroup):
    slug = "VIM_details"
    tabs = (VIMEventsTab,)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import ugettext_lazy as _
import yaml

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffgcatalog \
    import tables


class VNFFGCatalogItem(object):
    def __init__(self, name, description, vnffgd_id):
        self.id = vnffgd_id
        self.name = name
        self.description = description


class VNFFGCatalogTab(tabs.TableTab):
    name = _("VNFFGCatalog Tab")
    slug = "vnffgcatalog_tab"
    table_classes = (tables.VNFFGCatalogTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_vnffgcatalog_data(self):
        try:
            instances = []
            vnffgds = api.tacker.vnffgd_list(self.request)

            if len(vnffgds) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for vnffgd in vnffgds:
                item = VNFFGCatalogItem(vnffgd['name'],
                                        vnffgd['description'],
                                        vnffgd['id'])
                instances.append(item)
            return instances
        except Exception:
            self._has_more = False
            error_message = _('Unable to get instances')
            exceptions.handle(self.request, error_message)
            return []


class VNFFGCatalogTabs(tabs.TabGroup):
    slug = "vnffgcatalog_tabs"
    tabs = (VNFFGCatalogTab,)
    sticky = True


class TemplateTab(tabs.Tab):
    name = _("Template")
    slug = "template"
    template_name = ("nfv/vnffgcatalog/template.html")

    def get_context_data(self, request):
        vnffgd = self.tab_group.kwargs['vnffgd']
        vnffgd['template'] = yaml.safe_dump(vnffgd['template'],
                                            default_flow_style=False)
        return {'vnffgd': self.tab_group.kwargs['vnffgd']}


class VNFFGDDetailTabs(tabs.TabGroup):
    slug = "VNFFGD_details"
    tabs = (TemplateTab,)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from oslo_log import log as logging
from oslo_serialization import jsonutils

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffgmanager \
    import forms as project_forms

from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffgmanager \
    import tabs as nfv_tabs

LOG = logging.getLogger(__name__)


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = nfv_tabs.VNFFGManagerTabs
    template_name = 'nfv/vnffgmanager/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class DeployVNFFGView(forms.ModalFormView):
    form_class = project_forms.DeployVNFFG
    template_name = 'nfv/vnffgmanager/deploy_vnffg.html'
    success_url = reverse_lazy("horizon:nfv:vnffgmanager:index")
    modal_id = "deploy_vnffg_modal"
    modal_header = _("Deploy VNFFG")
    submit_label = _("Deploy VNFFG")
    submit_url = "horizon:nfv:vnffgmanager:deployvnffg"

    # @memoized.memoized_method
    # def get_object(self):
    #    try:
    #        return api.nova.server_get(self.request,
    #                                   self.kwargs["instance_id"])
    #    except Exception:
    #        exceptions.handle(self.request,
    #                          _("Unable to retrieve instance."))

    def get_initial(self):
        # return {"instance_id": self.kwargs["instance_id"]}
        return {}

    def get_context_data(self, **kwargs):
        context = super(DeployVNFFGView, self).get_context_data(**kwargs)
        # instance_id = self.kwargs['instance_id']
        # context['instance_id'] = instance_id
        # context['instance'] = self.get_object()
        context['submit_url'] = reverse(self.submit_url)
        return context


class DetailView(tabs.TabView):
    tab_group_class = nfv_tabs.VNFFGDetailsTabs
    template_name = 'nfv/vnffgmanager/detail.html'
    redirect_url = 'horizon:nfv:vnffgmanager:index'
    page_title = _("VNFFG Details: {{ vnffg_id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vnffg = self.get_data()
        context['vnffg'] = vnffg
        context['vnffg_id'] = kwargs['vnffg_id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        vnffg_id = self.kwargs['vnffg_id']

        try:
            vnffg = tacker_api.tacker.get_vnffg(self.request, vnffg_id)
            vnffg["vnffg"]["mgmt_ip_address"] = jsonutils.loads(
                vnffg["vnffg"]["mgmt_ip_address"]) if vnffg["vnffg"].get(
                "mgmt_ip_address") else None
            return vnffg
        except ValueError as e:
            msg = _('Cannot decode json : %s') % e
            LOG.error(msg)
        except KeyError as err:
            msg = _('Required field %s not specified') % err.args[0]
            LOG.error(msg)
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Unable to retrieve details for '
                                'VNFFG "%s".') % vnffg_id,
                              redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        vnffg = self.get_data()
        return self.tab_group_class(request, vnffg=vnffg, **kwargs)
